//-----------------------------------------------------------------------------
//  Drag & Drop Component
//  Copyright (c) 2003, 04 By Mental Studio -- http://mental.mentsu.com
//  Author: Raptor.z<raptor@mentsu.com>
//  Date: Dec.03-03, Dec.13, Dec.27, Jan.24-04, Feb.20, Dec.04
//---------------------------------------------------------------------------

#ifndef MOleDragDropH
#define MOleDragDropH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Classes.hpp>
#include <utilcls.h>

#ifndef CFSTR_SHELLURL
#define CFSTR_SHELLURL TEXT("UniformResourceLocator")
#endif

#include <oleidl.h>

#include "MCommon.h"
//---------------------------------------------------------------------------

namespace Moledragdrop
{

enum TMDropType { dtText, dtUnicodeText, dtUrl, dtHtml, dtFiles, dtBitmap };

typedef Set<TMDropType, dtText, dtBitmap>  TMDropTypes;

typedef Graphics::TBitmap TGBitmap;

//-- type declarations -------------------------------------------------------
typedef void __fastcall (__closure *TOleDragEnterEvent)(System::TObject* Sender,
    IDataObject * aDataObj, DWORD grfKeyState, _POINTL pt, DWORD *pdwEffect,
    tagFORMATETC &feFormat, const TMDropTypes aDropTypes);
typedef void __fastcall (__closure *TOleDragOverEvent)(System::TObject* Sender,
    DWORD grfKeyState, _POINTL pt, DWORD *pdwEffect, const TMDropTypes aDropTypes);
typedef void __fastcall (__closure *TOleDragLeaveEvent)(System::TObject* Sender);
typedef void __fastcall (__closure *TOleDragDropEvent)(System::TObject* Sender,
    IDataObject * aDataObj, DWORD grfKeyState, _POINTL pt, DWORD *pdwEffect,
    tagFORMATETC &feFormat);
typedef void __fastcall (__closure *TOleSimpleDropEvent)(System::TObject* Sender,
    IDataObject * aDataObj, DWORD grfKeyState, _POINTL pt, DWORD *pdwEffect,
    const TMDropTypes aDropTypes );

//---------------------------------------------------------------------------

class TMOleDragDrop;

class TMOleDropTarget : public System::TObject, public IDropTarget
{
private:
    ULONG           FRefCount;
	TMOleDragDrop * FOwner;
	HRESULT         FCanDrop;
	HRESULT         FDragResult;
	tagFORMATETC    FFormatEtc;

    void __fastcall CopyDIBToBitmap( Graphics::TBitmap * aBitmap, PBitmapInfo aBitmapInfo, int aDIBSize );

protected:
    //( Iunkown }
    HRESULT __stdcall QueryInterface( REFIID iid, void ** ppvObject );
    ULONG   __stdcall AddRef( );
    ULONG   __stdcall Release( );
    //{ IDropTarget }
    HRESULT __stdcall DragEnter(IDataObject *aDataObj,
        DWORD grfKeyState, _POINTL pt, DWORD *pdwEffect);
    HRESULT __stdcall DragOver(DWORD grfKeyState, _POINTL pt,
        DWORD *pdwEffect);
    HRESULT __stdcall DragLeave();
    HRESULT __stdcall Drop(IDataObject *aDataObj, DWORD grfKeyState,
        _POINTL pt, DWORD *pdwEffect);

public:
	__fastcall TMOleDropTarget(TMOleDragDrop* aOwner);
	__fastcall virtual ~TMOleDropTarget(void);

    AnsiString __fastcall GetDataTextA( IDataObject * aDataObj, tagFORMATETC * aFmtEtc );
    WideString __fastcall GetDataTextW( IDataObject * aDataObj, tagFORMATETC * aFmtEtc );
	int        __fastcall GetDataFiles( IDataObject * aDataObj, tagFORMATETC * aFmtEtc,
        Classes::TStrings* aFiles );
    bool       __fastcall GetDataBitmap( IDataObject * aDataObj, tagFORMATETC * aFmtEtc,
        Graphics::TBitmap * aBitmap );
    
	__property HRESULT CanDrop    = {read=FCanDrop};
	__property HRESULT DragResult = {read=FDragResult, write=FDragResult};
};

class PACKAGE TMOleDragDrop : public Classes::TComponent
{
    friend TMOleDropTarget;

private:
	TMOleDropTarget       * FOleDrag;
	Controls::TWinControl * FDropControl;
	bool                    FEnabledTarget;
    bool                    FAnsicode;

    TMDropTypes           FDropTypes;

	TOleDragEnterEvent    FOnDragEnter;
	TOleDragOverEvent     FOnDragOver;
	Classes::TNotifyEvent FOnDragLeave;
	TOleDragDropEvent     FOnDragDrop;

    TOleSimpleDropEvent   FOnSimpleDrop;

	void __fastcall SetDropControl(Controls::TWinControl* aValue);
	void __fastcall SetEnabledTarget(bool aValue);

protected:
	virtual void __fastcall Loaded(void);
	virtual void __fastcall Notification(Classes::TComponent* aComponent, Classes::TOperation aOperation);

    bool __fastcall GetDropData( IDataObject * aDataObj, TMDropType aDropType, tagFORMATETC * aFmtEtc );

public:
	__fastcall TMOleDragDrop(Classes::TComponent* AComponent);
	__fastcall virtual ~TMOleDragDrop(void);

    bool __fastcall GetDropData( IDataObject * aDataObj, TMDropType aDropType, AnsiString &aText );
    bool __fastcall GetDropData( IDataObject * aDataObj, TMDropType aDropType, WideString &aText );
    bool __fastcall GetDropData( IDataObject * aDataObj, TMDropType aDropType, TStrings * aFiles );
    bool __fastcall GetDropData( IDataObject * aDataObj, TMDropType aDropType, TGBitmap * aBitmap );
    //__property TMOleDropTarget * OleDragTarget = { read=FOleDrag };

    /* /  Only for debug
    void __fastcall CheckRegister( void )
    {
        if ( FDropControl )
        {
            OLECHECK( ::RevokeDragDrop( FDropControl->Handle ) );
            OLECHECK( ::RegisterDragDrop( FDropControl->Handle, FOleDrag ) );
        }
    }*/

__published:
	__property Controls::TWinControl* DropControl = {read=FDropControl, write=SetDropControl};
	__property bool EnabledTarget = {read=FEnabledTarget, write=SetEnabledTarget, default=1};
    __property bool Ansicode  = { read=FAnsicode, write=FAnsicode, default=1};
	__property TOleDragEnterEvent OnDragEnter = {read=FOnDragEnter, write=FOnDragEnter};
	__property TOleDragOverEvent OnDragOver = {read=FOnDragOver, write=FOnDragOver};
	__property Classes::TNotifyEvent OnDragLeave = {read=FOnDragLeave, write=FOnDragLeave};
	__property TOleDragDropEvent OnDragDrop = {read=FOnDragDrop, write=FOnDragDrop};
    __property TOleSimpleDropEvent OnSimpleDrop = {read=FOnSimpleDrop, write=FOnSimpleDrop};
};

//---------------------------------------------------------------------------
//   DropControlListViewʱıViewStyleRegisterDragDropʧЧ
//   ҪڸıViewStyleǰDropControlıٻָṩ෽ʹ

class TTempSaveOleDragDropControl
{
private:
    TMOleDragDrop         * FOwner;
    Controls::TWinControl * FControl;

public:
    __fastcall TTempSaveOleDragDropControl( TMOleDragDrop * aOwner )
        : FOwner( aOwner )
    {
        FControl = FOwner->DropControl;
        FOwner->DropControl = NULL;
    }
    __fastcall ~TTempSaveOleDragDropControl( )
    {
        FOwner->DropControl = FControl;
    }
};

//---------------------------------------------------------------------------

extern int CF_URL;
extern int CF_HTML;

//---------------------------------------------------------------------------
}	/* namespace Moledragdrop */
using namespace Moledragdrop;
//---------------------------------------------------------------------------
#endif
 